<?php

/**
 * Model For Access Log Creation And Update
 *
 * @author Softpro India Pvt. Ltd.
 */
defined('BASEPATH') OR exit('No direct script access allowed');

class AccessLog extends CI_Model {

    function createAccessLog($id, $loginStatus, $ip, $browserInfo) {
        $accessLogInfo = array(
            'access_ip' => $ip,
            'access_browser_details' => $browserInfo,
            'access_login_status' => $loginStatus,
            'access_active_status' => 'T',
            'access_auth_id' => $id,
            'access_login_at' => date("Y-m-d H:i:s"),
            'access_session_id' => $this->session->session_id
        );
        $this->db->insert('tbl_access_log', $accessLogInfo);
        return $this->db->insert_id();
    }

    function updateAccessLog($accessId, $loginActiveStatus) {
        $accessLogInfo = array(
            'access_active_status' => $loginActiveStatus,
            'access_logout_at' => date("Y-m-d H:i:s")
        );
        $this->db->where('access_id', $accessId);
        $this->db->update('tbl_access_log', $accessLogInfo);
    }

    function getAllAccessLogsBy($smember_id) {
        $this->db->select("*");
        $this->db->from('tbl_access_log TAL');
        $this->db->where("TAL.access_auth_id", $smember_id);
        $this->db->order_by("TAL.access_login_at", "desc");
        return $this->db->get();
    }

    function getSuccessfulLoginCountsBy($smember_id) {
        $this->db->select("count(*) totalLogins");
        $this->db->from('tbl_access_log TAL');
        $this->db->where("TAL.access_auth_id", $smember_id);
        $this->db->where("TAL.access_login_status", "T");
        return $this->db->get();
    }

    function deleteAccessLogsBy($smember_id) {
        $this->db->where("access_auth_id", $smember_id);
        $this->db->delete("tbl_access_log");
    }

    function getAccessLogsByDate($date, $role = '', $smember_id = '') {
        $this->db->select("DISTINCT(TAL.access_auth_id), TSM.*, TPRFL.*,TLD.*,TRM.*,TDEM.*,DCM.*,TSDM.*,TDM.*");
        $this->db->from('tbl_access_log TAL');
        $this->db->join('tbl_staff_members TSM', 'TAL.access_auth_id = TSM.smember_id');
        $this->db->join('tbl_profile TPRFL', 'TSM.tprfl_id = TPRFL.tprfl_id');
        $this->db->join('tbl_logindetails TLD', 'TSM.tld_id = TLD.tld_id');
        $this->db->join('tbl_role_master TRM', 'TSM.role_id = TRM.role_id');
        $this->db->join('tbl_designation_master TDEM', 'TPRFL.desig_id = TDEM.desig_id');
        $this->db->join('designation_category_mst DCM', 'TDEM.dcm_id = DCM.dcm_id');
        $this->db->join('tbl_sub_departments_master TSDM', 'TLD.sub_dept_id = TSDM.sub_dept_id');
        $this->db->join('tbl_department_master TDM', 'TSDM.dept_id = TDM.dept_id');
        if (trim($role) != "") {
            $this->db->where("TRM.role_id", $role);
        }
        if (trim($smember_id) != "") {
            $this->db->where("TAL.access_auth_id", $smember_id);
        }
        $this->db->where("DATE(TAL.access_login_at) = '" . $date . "'");
        return $this->db->get();
    }

}
