<?php

/**
 * Model For Handling All DB Operations Related To Exams
 *
 * @author Softpro India Pvt. Ltd.
 */
defined('BASEPATH') OR exit('No direct script access allowed');

class ExamManagement extends CI_Model {

    function createNewExam(array $newExamInfo) {
        $this->db->insert('tbl_exams_master', $newExamInfo);
        return $this->db->insert_id();
    }

    function getAllExams() {
        $this->db->select("TEM.exam_id,TEM.exam_name,TEM.exam_title,TEM.exam_description,TEM.exam_active_status,TEM.exam_delete_status,"
                . "TEM.exam_added_by,TEM.exam_added_on,TEM.exam_updated_by,TEM.exam_updated_on,TEM.exam_type_flag,"
                . "CONCAT(TPRFLA.tprfl_firstname,' ',TPRFLA.tprfl_lastname) addedByAdmin,"
                . "CONCAT(TPRFLU.tprfl_firstname,' ',TPRFLU.tprfl_lastname) updatedByAdmin");
        $this->db->from('tbl_exams_master TEM');
        $this->db->join('tbl_staff_members TSMA', 'TEM.exam_added_by = TSMA.smember_id');
        $this->db->join('tbl_profile TPRFLA', 'TSMA.smember_id = TPRFLA.tprfl_id');
        $this->db->join('tbl_staff_members TSMU', 'TEM.exam_updated_by = TSMU.smember_id');
        $this->db->join('tbl_profile TPRFLU', 'TSMU.smember_id = TPRFLU.tprfl_id');
        $this->db->order_by("TEM.exam_updated_on", "desc");
        return $this->db->get();
    }

    function getExamInfoBy($exam_id) {
        $this->db->select("TEM.exam_id,TEM.exam_name,TEM.exam_title,TEM.exam_description,TEM.exam_active_status,TEM.exam_delete_status,"
                . "TEM.exam_added_by,TEM.exam_added_on,TEM.exam_updated_by,TEM.exam_updated_on,TEM.exam_type_flag,"
                . "CONCAT(TPRFLA.tprfl_firstname,' ',TPRFLA.tprfl_lastname) addedByAdmin,"
                . "CONCAT(TPRFLU.tprfl_firstname,' ',TPRFLU.tprfl_lastname) updatedByAdmin");
        $this->db->from('tbl_exams_master TEM');
        $this->db->join('tbl_staff_members TSMA', 'TEM.exam_added_by = TSMA.smember_id');
        $this->db->join('tbl_profile TPRFLA', 'TSMA.smember_id = TPRFLA.tprfl_id');
        $this->db->join('tbl_staff_members TSMU', 'TEM.exam_updated_by = TSMU.smember_id');
        $this->db->join('tbl_profile TPRFLU', 'TSMU.smember_id = TPRFLU.tprfl_id');
        $this->db->where("TEM.exam_id", $exam_id);
        return $this->db->get();
    }

    function getExamInfoByName($exam_name) {
        $this->db->select("TEM.exam_id,TEM.exam_name,TEM.exam_title,TEM.exam_description,TEM.exam_active_status,TEM.exam_delete_status,"
                . "TEM.exam_added_by,TEM.exam_added_on,TEM.exam_updated_by,TEM.exam_updated_on,TEM.exam_type_flag,"
                . "CONCAT(TPRFLA.tprfl_firstname,' ',TPRFLA.tprfl_lastname) addedByAdmin,"
                . "CONCAT(TPRFLU.tprfl_firstname,' ',TPRFLU.tprfl_lastname) updatedByAdmin");
        $this->db->from('tbl_exams_master TEM');
        $this->db->join('tbl_staff_members TSMA', 'TEM.exam_added_by = TSMA.smember_id');
        $this->db->join('tbl_profile TPRFLA', 'TSMA.smember_id = TPRFLA.tprfl_id');
        $this->db->join('tbl_staff_members TSMU', 'TEM.exam_updated_by = TSMU.smember_id');
        $this->db->join('tbl_profile TPRFLU', 'TSMU.smember_id = TPRFLU.tprfl_id');
        $this->db->where("TEM.exam_name", $exam_name);
        return $this->db->get();
    }

    function getExamInfoByTitle($exam_title) {
        $this->db->select("TEM.exam_id,TEM.exam_name,TEM.exam_title,TEM.exam_description,TEM.exam_active_status,TEM.exam_delete_status,"
                . "TEM.exam_added_by,TEM.exam_added_on,TEM.exam_updated_by,TEM.exam_updated_on,TEM.exam_type_flag,"
                . "CONCAT(TPRFLA.tprfl_firstname,' ',TPRFLA.tprfl_lastname) addedByAdmin,"
                . "CONCAT(TPRFLU.tprfl_firstname,' ',TPRFLU.tprfl_lastname) updatedByAdmin");
        $this->db->from('tbl_exams_master TEM');
        $this->db->join('tbl_staff_members TSMA', 'TEM.exam_added_by = TSMA.smember_id');
        $this->db->join('tbl_profile TPRFLA', 'TSMA.smember_id = TPRFLA.tprfl_id');
        $this->db->join('tbl_staff_members TSMU', 'TEM.exam_updated_by = TSMU.smember_id');
        $this->db->join('tbl_profile TPRFLU', 'TSMU.smember_id = TPRFLU.tprfl_id');
        $this->db->where("TEM.exam_title", $exam_title);
        return $this->db->get();
    }

    function getAllExamsByCategory($category) {
        $this->db->select("TEM.exam_id,TEM.exam_name,TEM.exam_title,TEM.exam_description,TEM.exam_active_status,TEM.exam_delete_status,"
                . "TEM.exam_added_by,TEM.exam_added_on,TEM.exam_updated_by,TEM.exam_updated_on,TEM.exam_type_flag");
        $this->db->from('tbl_exams_master TEM');
        $this->db->where("TEM.exam_active_status", "T");
        $this->db->where("TEM.exam_delete_status", "F");
        $this->db->where("TEM.exam_type_flag", $category);
        return $this->db->get();
    }

    function getAllExamsUptoCategory($category = '') {
        $this->db->select("TEM.exam_id,TEM.exam_name,TEM.exam_title,TEM.exam_description,TEM.exam_active_status,TEM.exam_delete_status,"
                . "TEM.exam_added_by,TEM.exam_added_on,TEM.exam_updated_by,TEM.exam_updated_on,TEM.exam_type_flag");
        $this->db->from('tbl_exams_master TEM');
        if ($category == "UG") {
            $this->db->where("TEM.exam_type_flag", "UG");
        } else {
            $this->db->where("TEM.exam_type_flag = 'UG' OR TEM.exam_type_flag = 'PG'");
        }
        $this->db->where("TEM.exam_active_status", "T");
        $this->db->where("TEM.exam_delete_status", "F");
        return $this->db->get();
    }

    function isExamNameSafeUpdate($exam_id, $exam_name) {
        $this->db->select("*");
        $this->db->from('tbl_exams_master');
        $this->db->where('exam_name', $exam_name);
        $this->db->where('exam_id != ' . $exam_id);
        $result = $this->db->get()->result();
        if (sizeof($result)) {
            return FALSE;
        } else {
            return TRUE;
        }
    }

    function isExamTitleSafeUpdate($exam_id, $exam_title) {
        $this->db->select("*");
        $this->db->from('tbl_exams_master');
        $this->db->where('exam_title', $exam_title);
        $this->db->where('exam_id != ' . $exam_id);
        $result = $this->db->get()->result();
        if (sizeof($result)) {
            return FALSE;
        } else {
            return TRUE;
        }
    }

    function updateExamInfo(array $examUpdatedInfo) {
        $this->db->where('exam_id', $examUpdatedInfo['exam_id']);
        return $this->db->update('tbl_exams_master', $examUpdatedInfo);
    }

}
