<?php

/**
 * Model Class For Handling All DB Operations Related To Courses
 *
 * @author Softpro India Pvt. Ltd.
 */
defined('BASEPATH') OR exit('No direct script access allowed');

class CourseManagement extends CI_Model {

    function __construct() {
        parent::__construct();
        $this->load->database();
    }

    function getTotalNonDeletedCoursesOfInstitute($clg_id) {
        $this->db->select("count(*) totalCourses");
        $this->db->from('course_mst');
        $this->db->where('clg_id', $clg_id);
        $this->db->where('course_delete_status', 'F');
        return $this->db->get();
    }

    function getNonDeletedCoursesOfInstitute($clg_id) {
        $this->db->select("CM.course_id,CM.course_name,CM.course_active_status,CM.course_delete_status,CM.course_full_name,"
                . "CM.course_no_of_years,CM.course_no_of_sems,CM.course_populate_flag,CM.course_added_on,CM.course_updated_on,"
                . "CAUA.cau_first_name addedByCA,CAUM.cau_first_name updatedByCA");
        $this->db->from('course_mst CM');
        $this->db->join('college_admins_and_users CAUA', 'CM.course_added_by = CAUA.cau_id');
        $this->db->join('college_admins_and_users CAUM', 'CM.course_updated_by = CAUM.cau_id');
        $this->db->where('CM.clg_id', $clg_id);
        $this->db->where('CM.course_delete_status', 'F');
        $this->db->order_by('CM.course_added_on desc');
        return $this->db->get();
    }
    
    function getNonDeletedActiveCoursesOfInstitute($clg_id){
        $this->db->select("CM.course_id,CM.course_name,CM.course_active_status,CM.course_delete_status,CM.course_full_name,"
                . "CM.course_no_of_years,CM.course_no_of_sems,CM.course_populate_flag,CM.course_added_on,CM.course_updated_on,"
                . "CAUA.cau_first_name addedByCA,CAUM.cau_first_name updatedByCA");
        $this->db->from('course_mst CM');
        $this->db->join('college_admins_and_users CAUA', 'CM.course_added_by = CAUA.cau_id');
        $this->db->join('college_admins_and_users CAUM', 'CM.course_updated_by = CAUM.cau_id');
        $this->db->where('CM.clg_id', $clg_id);
        $this->db->where('CM.course_active_status', 'T');
        $this->db->where('CM.course_delete_status', 'F');
        $this->db->order_by('CM.course_added_on desc');
        return $this->db->get();
    }

    function createNewCourse(array $newCourseInfo) {
        $this->db->insert('course_mst', $newCourseInfo);
        return $this->db->insert_id();
    }

    function updateCourseInfo(array $courseUpdatedInfo) {
        $this->db->where('course_id', $courseUpdatedInfo['course_id']);
        return $this->db->update('course_mst', $courseUpdatedInfo);
    }

    function getCourseInfoById($courseId) {
        $this->db->select("*");
        $this->db->from('course_mst');
        $this->db->where('course_id', $courseId);
        return $this->db->get();
    }

}
